<?php

declare(strict_types=1);

use Rector\Core\Configuration\Option;
use Rector\Php55\Rector\String_\StringClassNameToClassConstantRector;
use Rector\Set\ValueObject\LevelSetList;
use Rector\Set\ValueObject\SetList;
use Rector\Symfony\Set\SymfonySetList;
use Symfony\Component\DependencyInjection\Loader\Configurator\ContainerConfigurator;

return static function (ContainerConfigurator $containerConfigurator): void {
    $parameters = $containerConfigurator->parameters();

    $parameters->set(Option::AUTO_IMPORT_NAMES, true);
    $parameters->set(Option::PATHS, [
        __DIR__ . '/src',
        __DIR__ . '/tests',
    ]);

    // experimental
    $parameters->set(Option::PARALLEL, true);

    $parameters->set(Option::SKIP, [
        '*/Fixture/*',
        '*/Source/*',
        '*/Source*/*',
    ]);

    $services = $containerConfigurator->services();

    $services->set(StringClassNameToClassConstantRector::class)
        ->configure([
            'Symfony\*',
            'Twig_*',
            'Swift_*',
            'Doctrine\*',
        ]);

    $containerConfigurator->import(LevelSetList::UP_TO_PHP_81);
    $containerConfigurator->import(SetList::CODE_QUALITY);
    $containerConfigurator->import(SetList::DEAD_CODE);

    // for testing
    $containerConfigurator->import(__DIR__ . '/config/config.php');
    $containerConfigurator->import(SymfonySetList::SYMFONY_60);
};
